﻿using System.ComponentModel;
using Exam.Infrastructure.Extensions;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.BatchTask;

/// <summary>
/// 系统任务
/// </summary>
public class Apptask : CreationEntity
{
    /// <summary>
    /// 任务名称
    /// </summary>
    public string Name { get; set; }

    /// <summary>
    /// 任务描述
    /// </summary>
    public string? Description { get; set; }

    /// <summary>
    /// 任务类型
    /// </summary>
    public ETaskType TaskType { get; set; }

    [Navigate(NavigateType.OneToMany, nameof(ApptaskItem.ApptaskId))]
    public List<ApptaskItem> ApptaskItems { get; set; }

    public void CreateName() => Name = $"{TaskType.GetDescription()}-{DateTime.Now:yyyyMMddHHmmssfff}";
}


public class ApptaskItem
{
    /// <summary>
    /// 任务id
    /// </summary>
    public string ApptaskId { get; set; }

    /// <summary>
    /// 业务id（冗余）
    /// </summary>
    public string BusinessId { get; set; }

    /// <summary>
    /// 任务类型(冗余)
    /// </summary>
    public ETaskType TaskType { get; set; }

    /// <summary>
    /// 任务状态
    /// </summary>
    public ETaskStatus TaskStatus { get; set; }

    /// <summary>
    /// 任务执行时间
    /// </summary>
    public DateTime? TaskStartTime { get; set; }
    public DateTime? TaskEndTime { get; set; }

    /// <summary>
    /// 参数
    /// </summary>
    [SugarColumn(ColumnDataType = "varchar(8000)")]
    public string TaskParams { get; set; }

    /// <summary>
    /// 执行次数
    /// </summary>
    public int Tries { get; set; }

    /// <summary>
    /// 执行次数上限
    /// </summary>
    public int Limit { get; set; } = 1;

    [SugarColumn(IsEnableUpdateVersionValidation = true)]
    public string Ver { get; set; }

    [Navigate(NavigateType.OneToOne, nameof(ApptaskId))]
    public Apptask Apptask { get; set; }
}

/// <summary>
/// 任务状态
/// </summary>
public enum ETaskStatus
{
    Wait = 0,
    Processing = 1,
    Success = 2,
    Failed = 3,
    Terminated = 9,
}

/// <summary>
/// 任务类型
/// </summary>
public enum ETaskType
{
    /// <summary>
    /// 延期
    /// </summary>
    [Description("延期任务")]
    Delay = 1,

    /// <summary>
    /// 甄别
    /// </summary>
    [Description("甄别任务")]
    Screen = 2,
}